
package BlowfishJ;

import java.lang.*;


/**
 * testing the Blowfish encryption algorithm classes
 * @author Markus Hahn <markus_hahn@gmx.net>
 * @version January 30, 2003
 */

public class BlowfishTest 
{
    // max. size of message to encrypt

    final static int MAX_MESS_SIZE = 64;


    // benchmark settings

    final static int TESTBUFSIZE = 100000;
    final static int TESTLOOPS   = 10000;

         
    // startup CBC IV

    final static long CBCIV_START = 0x0102030405060708L;

        
    /**
     * the application entry point
     * @param args (command line) parameters
     */
    public static void main
		(String args[])
    {
        // first do the self test

        System.out.print("running self test...");
    
        if (!BlowfishECB.selfTest())
        {
            System.out.println(", FAILED");
            return;
        }
        
        System.out.println(", passed.");

        // now the classic examples...

        // create our test key

        byte[] testkey = new byte[5];
        for (int nI = 0; nI < testkey.length; nI++)
        {
            testkey[nI] = (byte)(nI + 1);
        }
         
        // do the key setups and check for weaknesses
    
        System.out.print("setting up Blowfish keys...");
     
        BlowfishECB bfecb = new BlowfishECB(testkey);
        
        BlowfishCBC bfcbc = new BlowfishCBC(testkey, CBCIV_START);
        
        System.out.println(", done.");
        
        if (bfecb.weakKeyCheck()) System.out.println("ECB key is weak!");
        else                      System.out.println("ECB key OK");

        if (bfcbc.weakKeyCheck()) System.out.println("CBC key is weak!");
        else                      System.out.println("CBC key OK");
            
        // get a message

        System.out.print("something to encrypt please >");
        System.out.flush();

        byte[] tempbuf = new byte[MAX_MESS_SIZE];

        int nMsgSize = 0;

        int nLnBrkLen = 0;
        try { nLnBrkLen = System.getProperty("line.separator").length(); } 
        catch (Throwable err) { };

        try 
        { 
            nMsgSize = System.in.read(tempbuf) - nLnBrkLen; // (cut off the line break)
            byte[] cpyBuf = new byte[nMsgSize];
            System.arraycopy(tempbuf, 0, cpyBuf, 0, nMsgSize);
            tempbuf = cpyBuf;
        }
        catch (java.io.IOException ioe) 
        { 
            return; 
        }

        // align to the next 8 byte border

        byte[] msgbuf;

        int nRest = nMsgSize & 7;

        if (nRest != 0) 
        {
            msgbuf = new byte[(nMsgSize & (~7)) + 8];

            System.arraycopy(tempbuf, 0, msgbuf, 0, nMsgSize);

            for (int nI = nMsgSize; nI < msgbuf.length ; nI++) 
            {
                msgbuf[nI] = 0;
            }
 
            System.out.println("message with " + nMsgSize + " bytes aligned to " + 
                               msgbuf.length + " bytes");
        }
        else 
        {
            msgbuf = new byte[nMsgSize];

            System.arraycopy(tempbuf, 0, msgbuf, 0, nMsgSize);
        }

        System.out.println("aligned data : " + BinConverter.bytesToBinHex(msgbuf));

        // ECB encryption/decryption test...

        bfecb.encrypt(msgbuf);

        // show the result

        System.out.println("ECB encrypted: " + BinConverter.bytesToBinHex(msgbuf));

        bfecb.decrypt(msgbuf);

        System.out.println("ECB decrypted: >>>" + new String(msgbuf) + "<<<");


        // CBC encryption/decryption test...

        byte[] showIV = new byte[BlowfishCBC.BLOCKSIZE];

        bfcbc.getCBCIV(showIV);

        System.out.println("CBC IV: " + BinConverter.bytesToBinHex(showIV));

        bfcbc.encrypt(msgbuf);


        // show the result

        System.out.println("CBC encrypted: " + BinConverter.bytesToBinHex(msgbuf));

        bfcbc.setCBCIV(CBCIV_START);
        bfcbc.decrypt(msgbuf);

        System.out.println("CBC decrypted: >>>" + new String(msgbuf) + "<<<");

        // test official vectors ...
        
        int nI = 0, nJ;
        
        while (nI < TEST_DATA.length)
        {
            byte[] key      = new byte[8];        long lKey     = TEST_DATA[nI++];
            byte[] plain  = new byte[8];        long lPlain     = TEST_DATA[nI++];
            byte[] cipher = new byte[8];        long lCipher = TEST_DATA[nI++];

            for (nJ = 7; nJ >= 0; nJ--)
            {
                key     [nJ]  = (byte)(lKey    & 0x0ff);    lKey    >>>= 8;        
                plain [nJ] = (byte)(lPlain    & 0x0ff);    lPlain    >>>= 8;    
                cipher[nJ] = (byte)(lCipher & 0x0ff);    lCipher >>>= 8;         
            }
                        
            byte[] testBuf = new byte[8];

            bfecb = new BlowfishECB(key);

            bfecb.encrypt(plain, testBuf);

            for (nJ = 0; nJ < 8; nJ++)
            {
                if (testBuf[nJ] != cipher[nJ])
                {
                    System.out.println("error on vector " + nI / 3);
                }
            }
        }
        
        System.out.println("tests done.");
        
        // benchmark...

        System.out.println("\nbenchmark is running, encrypting " + 
                           TESTBUFSIZE * TESTLOOPS + " bytes\nplease wait...");    

        long lTm = System.currentTimeMillis();

        byte[] testbuf = new byte[TESTBUFSIZE];

        for (nI = 0; nI < TESTLOOPS; nI++) 
        {
            bfecb.encrypt(testbuf);

            if ((nI % 0x0f) == 0) 
            {
                 System.out.print("#");
                 System.out.flush();
            }        
        }

        lTm = System.currentTimeMillis() - lTm;

        System.out.println();

        double dAmount = TESTBUFSIZE * TESTLOOPS;
        double dTime = lTm;
        double fRate = (dAmount * 1000) / dTime;
        long lRate = (long) fRate;

        System.out.println("rate: " + lRate + " bytes/sec");

        
        // don't leave any traces in the memory

        bfecb.cleanUp();
        bfcbc.cleanUp();
    }


    // (the official test vectors from the Counterpane website)

    final static long[] TEST_DATA = 
    {
        0x0000000000000000L, 0x0000000000000000L, 0x4ef997456198dd78L,
        0xffffffffffffffffL, 0xffffffffffffffffL, 0x51866fd5b85ecb8aL,
        0x3000000000000000L, 0x1000000000000001L, 0x7d856f9a613063f2L,
        0x1111111111111111L, 0x1111111111111111L, 0x2466dd878b963c9dL,
        0x0123456789abcdefL, 0x1111111111111111L, 0x61f9c3802281b096L,
        0x1111111111111111L, 0x0123456789abcdefL, 0x7d0cc630afda1ec7L,
        0x0000000000000000L, 0x0000000000000000L, 0x4ef997456198dd78L,
        0xfedcba9876543210L, 0x0123456789abcdefL, 0x0aceab0fc6a0a28dL,
        0x7ca110454a1a6e57L, 0x01a1d6d039776742L, 0x59c68245eb05282bL,
        0x0131d9619dc1376eL, 0x5cd54ca83def57daL, 0xb1b8cc0b250f09a0L,
        0x07a1133e4a0b2686L, 0x0248d43806f67172L, 0x1730e5778bea1da4L,
        0x3849674c2602319eL, 0x51454b582ddf440aL, 0xa25e7856cf2651ebL,
        0x04b915ba43feb5b6L, 0x42fd443059577fa2L, 0x353882b109ce8f1aL,
        0x0113b970fd34f2ceL, 0x059b5e0851cf143aL, 0x48f4d0884c379918L,
        0x0170f175468fb5e6L, 0x0756d8e0774761d2L, 0x432193b78951fc98L,
        0x43297fad38e373feL, 0x762514b829bf486aL, 0x13f04154d69d1ae5L,
        0x07a7137045da2a16L, 0x3bdd119049372802L, 0x2eedda93ffd39c79L,
        0x04689104c2fd3b2fL, 0x26955f6835af609aL, 0xd887e0393c2da6e3L,
        0x37d06bb516cb7546L, 0x164d5e404f275232L, 0x5f99d04f5b163969L,
        0x1f08260d1ac2465eL, 0x6b056e18759f5ccaL, 0x4a057a3b24d3977bL,
        0x584023641aba6176L, 0x004bd6ef09176062L, 0x452031c1e4fada8eL,
        0x025816164629b007L, 0x480d39006ee762f2L, 0x7555ae39f59b87bdL,
        0x49793ebc79b3258fL, 0x437540c8698f3cfaL, 0x53c55f9cb49fc019L,
        0x4fb05e1515ab73a7L, 0x072d43a077075292L, 0x7a8e7bfa937e89a3L,
        0x49e95d6d4ca229bfL, 0x02fe55778117f12aL, 0xcf9c5d7a4986adb5L,
        0x018310dc409b26d6L, 0x1d9d5c5018f728c2L, 0xd1abb290658bc778L,
        0x1c587f1c13924fefL, 0x305532286d6f295aL, 0x55cb3774d13ef201L,
        0x0101010101010101L, 0x0123456789abcdefL, 0xfa34ec4847b268b2L,
        0x1f1f1f1f0e0e0e0eL, 0x0123456789abcdefL, 0xa790795108ea3caeL,
        0xe0fee0fef1fef1feL, 0x0123456789abcdefL, 0xc39e072d9fac631dL,
        0x0000000000000000L, 0xffffffffffffffffL, 0x014933e0cdaff6e4L,
        0xffffffffffffffffL, 0x0000000000000000L, 0xf21e9a77b71c49bcL,
        0x0123456789abcdefL, 0x0000000000000000L, 0x245946885754369aL,
        0xfedcba9876543210L, 0xffffffffffffffffL, 0x6b5c5a9c5d9e0a5aL
    };
   
}

